#include <QtGui>
#include <QPainter>
#include <QPushButton>
#include <QPaintEvent>
#include <QThread>
#include "mainwindowimpl.h"
#include "displaywidget.h"											//QDevelop
#include "commthread.h"											//QDevelop
#include "dso.h"											//QDevelop

Dso	*dso;

#include	"build/ui_mainwindow.h"

MainWindowImpl::MainWindowImpl( QWidget * parent, Qt::WFlags f) 	//QDevelop
	: QMainWindow(parent, f)										//QDevelop
{
	dso = new Dso;
	printf("%s (%s:%d)\n",__func__,__FILE__,__LINE__);
// Erzeuge das DisplayWidget, in das die Kurven gezeichnet werden.
	DisplayWidget *displayWidget = new DisplayWidget(this);
	displayWidget->show();

//Create communications-Thread:

	setupUi(this);													//QDevelop

	getDataButton->setText("Start");

	setUpSliders();
	setUpMenu();

	dso->dsofd = dso->connect("/dev/ttyUSB0");
	commThread=new CommThread(displayWidget);
	connect(commThread, SIGNAL(started()), this ,SLOT(thStarted()));
	connect(commThread, SIGNAL(finished()), this ,SLOT(thStopped()));
	connect(getDataButton , SIGNAL(clicked()), commThread, SLOT(thControl()));
	connect(commThread, SIGNAL(serialDataRx(int, unsigned char *)), this, SLOT(serialDataRx(int, unsigned char *)));

	editor = new QTextEdit(0);
	editor->setReadOnly(true);
	editor->show();
}

void MainWindowImpl::serialDataRx(int count, unsigned char *buffer) {
QString text;

	printf("%s (%s:%d)\n",__func__,__FILE__,__LINE__);
	for (int idx=0; idx<count; idx++) {
		if (isprint(buffer[idx])) {
			text.append(buffer[idx]); }	
		else text.append(QString("<%1>").arg(buffer[idx],0,16));
	}
	editor->append(text);
}

void MainWindowImpl::setUpMenu() {
	printf("%s (%s:%d)\n",__func__,__FILE__,__LINE__);

	quitAction = new QAction(tr("&Quit..."), this);
	quitAction->setShortcut(tr("Ctrl+Q"));
	quitAction->setStatusTip(tr("Exit application"));

	openAction = new QAction(tr("&Open..."), this);
	openAction->setShortcut(tr("Ctrl+O"));
	openAction->setStatusTip(tr("Open an existing file"));

	fileMenu = menuBar()->addMenu(tr("&File"));
	fileMenu->addAction(openAction);
	fileMenu->addAction(quitAction);

	editPrefsAction = new QAction(tr("&Preferences..."), this);
	editPrefsAction->setShortcut(tr("Ctrl+P"));
	editPrefsAction->setStatusTip(tr("Edit preferences"));

	editMenu = menuBar()->addMenu(tr("&Edit"));
	editMenu->addAction(editPrefsAction);

	connect(openAction, SIGNAL(triggered()), this, SLOT(open()));
	connect(quitAction, SIGNAL(triggered()), this, SLOT(quit()));
	connect(editPrefsAction, SIGNAL(triggered()), this, SLOT(editPrefs()));
}

void MainWindowImpl::setUpSliders() {
	ch1VertSlider->setTickInterval(1);
	ch1VertSlider->setTickPosition(QSlider::TicksBothSides);
	ch2VertSlider->setTickInterval(1);
	ch2VertSlider->setTickPosition(QSlider::TicksBothSides);

	timeBaseSlider->setMinimum(0);
	timeBaseSlider->setMaximum(dso->getNumberTimeBases()-1);
	timeBaseSlider->setTickInterval(1);
	timeBaseSlider->setTickPosition(QSlider::TicksBothSides);
	timeBaseSlider->setValue(dso->getNumberTimeBases()/2);

	connect(quitButton , SIGNAL(clicked()), this, SLOT(quit()));
	connect(ch1VertSlider, SIGNAL(valueChanged(int)), this ,SLOT(vertResChange1(int)));
	connect(ch2VertSlider, SIGNAL(valueChanged(int)), this ,SLOT(vertResChange2(int)));
	connect(timeBaseSlider, SIGNAL(valueChanged(int)), this ,SLOT(timeBaseChanged(int)));
	connect(testButton, SIGNAL(clicked()), this, SLOT(setNeedSettings()));
}

void	MainWindowImpl::setNeedSettings() {
	dso->setNeedSettings(); }

void	MainWindowImpl::editPrefs() {
	printf("%s (%s:%d)\n",__func__,__FILE__,__LINE__);
}

int		MainWindowImpl::open() {
QString	fileName;
	printf("%s (%s:%d)\n",__func__,__FILE__,__LINE__);

	QFileDialog	*fileDialog = new QFileDialog;
	fileDialog->setFileMode(QFileDialog::AnyFile);
	fileDialog->setViewMode(QFileDialog::Detail);
	fileDialog->setDirectory("/dev");

	fileDialog->exec();

//	fileName = fileDialog->getOpenFileName(this,"A", "/dev",
//		"(ttyUSB* ttyS*);;All files (*)");

//	fileName = QFileDialog::getOpenFileName(this,
//       tr("Open Image"), "/dev", tr("(ttyUSB* ttyS*);;Text files (*.txt)"));
}

void	MainWindowImpl::quit() {
	printf("%s (%s:%d)\n",__func__,__FILE__,__LINE__);
	exit(0);
}

void	MainWindowImpl::thStopped() {
	printf("%s (%s:%d)\n",__func__,__FILE__,__LINE__);
	getDataButton->setText("Start");
}

void	MainWindowImpl::thStarted() {
	printf("%s (%s:%d)\n",__func__,__FILE__,__LINE__);
	getDataButton->setText("Stop");
}

void	MainWindowImpl::vertResChange1(int value) {
	printf("%s (%s:%d)\n",__func__,__FILE__,__LINE__);
	dso->setVertSens(1,value);
	ch1VertLabel->setText(dso->getVertSensStr(value));
}

void	MainWindowImpl::vertResChange2(int value) {
	printf("%s (%s:%d)\n",__func__,__FILE__,__LINE__);
	dso->setVertSens(2,value);
	ch2VertLabel->setText(dso->getVertSensStr(value));
}

void	MainWindowImpl::timeBaseChanged(int value) {
	printf("%s (%s:%d)\n",__func__,__FILE__,__LINE__);
	dso->setTimeBase(value);
	timeBaseLabel->setText(dso->getTimBaseText(value));
}
